from secrets import randbits
from math import ceil

# Encrypted flag, see decrypt()
CIPHERTEXT = [92, 183, 208, 206, 165, 183, 124, 248, 172, 149, 184, 157, 146, 191, 149, 15, 200, 165, 87, 55, 68, 161, 22, 52, 188, 92, 60, 64, 110]

def split(secret, shares, threshold):
    coefficients = [randbits(8*len(CIPHERTEXT)) for _ in range(threshold - 1)]
    def solve(x, i):
        if i == -1:
            return secret
        return x**(i + 1) * coefficients[i] + solve(x, i-1)
    def f(x):
        return solve(x, len(coefficients) - 1)
    # Start at x+1 because f(0) would be the secret.
    return [f(x+1) for x in range(shares)]

def decrypt(secret):
    """This function implements a simple yet secure one time pad encryption. It
    is not necessary to understand or attack this function to get the flag."""
    plaintext = b""
    for i in range(ceil(secret.bit_length() / 8)):
        key = secret >> (i * 8) & 0xff
        plaintext += bytes([CIPHERTEXT[i] ^ key ])
    return plaintext

MESSAGE = """
Hey, I have a secret number that I want to share with my friends, but I want to
make sure that they have to be together to get the secret. Specifically, I've
split my secret in to 5 shares and given one to each friend. Any 3 of them
should be able to recover the secret.

Here's the code I ran:

  split(<secret>, 5, 3)

Which returned these 5 shares:

    7645951121772129837715091730450114347403374755900653665061720589889378
    28142261357114423733518280210932526229339105547822046647771484014677103
    62013527918813884743557193115025241204738535569231638501028524127763014
    109259750806870512867831830442728259273601664820129429224832840929147111
    169880930021284308106342192194041580435928493300515418819184434418829394

Before I hand these out to my friends, I want to make sure that my code works.
Given all 5 of the shares, can you determine the secret?
"""

if __name__ == "__main__":
    print(MESSAGE)
    try:
        flag = decrypt(int(input("I think the secret is: "))).decode()
    except UnicodeDecodeError:
        print("Hmm, not quite right, but I'm sure my code works!")
    else:
        print("That's right! Here's the flag:")
        print(flag)
